#!/usr/bin/env python

"""buildbunker uses boto3/ssm to clone all your git repos and then rsync up dot files and configs"""

import configparser
import sys
import os
from os.path import expanduser
import time
import subprocess
import boto3

class Bcolors:
    CYAN = '\033[96m'
    MAGENTA = '\033[95m'
    GREY = '\033[90m'
    OKBLUE = '\033[94m'
    OKGREEN = '\033[92m'
    YELLOW = '\033[33m'
    RED = '\033[31m'
    WARNING = '\033[93m'
    FAIL = '\033[91m'
    ENDC = '\033[0m'
    BOLD = '\033[1m'
    UNDERLINE = '\033[4m'
    ORANGE = '\033[38;5;208m'
    PINK = '\033[38;5;212m'
    PALEYELLOW = '\033[38;5;228m'
    PALEBLUE = '\033[38;5;111m'
    GOLD = '\033[38;5;178m'

def query_yes_no(question, default="yes"):
    '''confirm or decline'''
    valid = {"yes": True, "y": True, "ye": True, "no": False, "n": False}
    if default is None:
        prompt = " [y/n] "
    elif default == "yes":
        prompt = " [Y/n] "
    elif default == "no":
        prompt = " [y/N] "
    else:
        raise ValueError("invalid default answer: '%s'" % default)
    while True:
        sys.stdout.write(question + prompt)
        choice = input().lower()
        if default is not None and choice == '': 
            return valid[default]
        elif choice in valid:
            return valid[choice]
        else:
            sys.stdout.write("\nPlease respond with 'yes' or 'no' (or 'y' or 'n').\n")

def countdown(num):
    while num > 0:
        fmtnum = str(num)
        if num < 10:
            fmtnum = "0"+str(num)
        print(fmtnum, end="\r", flush=True)
        num -= 1
        time.sleep(1)
    return

def buildbunker(running=None, dostart=None, source=None, yes=None):
    """clone git repos on ec2, then copy over ignored files"""
    #config here
    home = expanduser('~')
    bunker_config = os.path.join(home, ".config-bunker.ini")
    config = configparser.ConfigParser()
    config.read(bunker_config)
    prefix = config['default']['prefix']
    repos = prefix+config['default']['repos']
    ignored = prefix+config['default']['ignored']
    pem = config['default']['pem']
    instance_id = config['default']['instance_id']
    username = config['default']['username']
    gitkeys = config['default']['gitkeys']
    gitkeys_arr = []
    keycount = gitkeys.count(', ')
    if keycount > 0:
        gitkeys_arr = config['default']['gitkeys'].split(", ")
    else:
        gitkeys_arr.append(config['default']['gitkeys'])
    # ec2 ssm client
    client = boto3.client('ec2')
    ssm_client = boto3.client('ssm')
    # start up if necessary
    if dostart:
        print(Bcolors.ENDC+"starting ec2 and waiting for \"running\" state...")
        client.start_instances(InstanceIds=[instance_id])
        waiter = client.get_waiter('instance_running')
        waiter.wait(InstanceIds=[instance_id])
        countdown(20)
        print("instance up: "+str(instance_id))
    # get ip address
    instance_ip = subprocess.run("aws ec2 describe-instances --instance-ids "+instance_id+" --query 'Reservations[*].Instances[*].PublicIpAddress' --output text", shell=True, stdout=subprocess.PIPE)
    instance_ip = instance_ip.stdout.decode('utf-8')
    instance_ip = instance_ip.strip(' \t\n\r')
    # get instance state
    state = 0
    state = subprocess.run("aws ec2 describe-instances --instance-ids "+instance_id+" --query 'Reservations[*].Instances[*].State.Code' --output text", shell=True, stdout=subprocess.PIPE)
    state = state.stdout.decode('utf-8')
    try:
        state = int(state)
    except ValueError:
        print(Bcolors.RED+"instance does not seem to exist..."+Bcolors.ENDC)
        print("ensure everything is set correctly by running: "+Bcolors.PINK+"bunker init"+Bcolors.ENDC)
        print()
        exit()
    state = int(state)
    if state != 16:
        print(Bcolors.RED+"instance is not running."+Bcolors.ENDC)
        if not running:
            stop = subprocess.run("aws ec2 stop-instances --instance-ids "+instance_id, shell=True, stdout=subprocess.PIPE)
        exit()
    # INSTANCE READY FOR WORK, DO STUFF
    #
    # auf gehts
    if yes or query_yes_no("ready to transfer to "+instance_ip+"?", "yes"):
        dirs = []
        diffs = []
        similarities = []
        if source:
            if source[-1] == "/":
                source = source[:-1]
            dirs.append(source)
        else:
            # build some arrays (dirs, diffs, similarities) to use later
            with open(repos, 'r') as f:
                x = 0
                for line in f:
                    repo = line.strip()
                    if repo[-1] == "/":
                        repo = repo[:-1]
                    dirs.append(repo)
                    exploded_repo = repo.split("/")
                    for directory in exploded_repo:
                        if x == 0 and directory != "":
                            diffs.append(directory)
                        else:
                            if directory in diffs and directory not in similarities:
                                similarities.append(directory)
                    x += 1

        # check for known_hosts
        response = ssm_client.send_command(
            InstanceIds=[instance_id],
            DocumentName="AWS-RunShellScript",
            Parameters={'commands': ['ls /home/'+username+'/.ssh/known_hosts']}, )
        check_id = response['Command']['CommandId']
        time.sleep(1)
        checkoutput = ssm_client.get_command_invocation(
            CommandId=check_id,
            InstanceId=instance_id,
        )   
        while checkoutput['Status'] != "Success" and checkoutput['Status'] != "Failed":
            time.sleep(1)
            checkoutput = ssm_client.get_command_invocation(
                CommandId=check_id,
                InstanceId=instance_id,
            )   
        if checkoutput['Status'] != "Success":
            print(Bcolors.WARNING+"no "+Bcolors.ORANGE+"known_hosts"+Bcolors.WARNING+" in /home/"+username+"/.ssh/"+Bcolors.ENDC)
            print("copy local known_hosts to EC2")
            if os.path.isfile(os.path.join(home, ".ssh", "known_hosts")):
                known_hosts = os.path.join(home, ".ssh", "known_hosts")
                cmd = "scp -i "+pem+" "+known_hosts+" "+username+"@"+instance_ip+":/home/"+username+"/.ssh/"
                print(Bcolors.OKGREEN+cmd+Bcolors.ENDC)
                subprocess.call(cmd, shell=True)
            else:
                print(Bcolors.WARNING+"no known_hosts in ~/.ssh/"+Bcolors.ENDC)
                print("please configure your local ssh to connect to your git providers before using bunker.")
                print()
                exit()

        # check for config
        response = ssm_client.send_command(
            InstanceIds=[instance_id],
            DocumentName="AWS-RunShellScript",
            Parameters={'commands': ['ls /home/'+username+'/.ssh/config']}, )
        check_id = response['Command']['CommandId']
        time.sleep(1)
        checkoutput = ssm_client.get_command_invocation(
            CommandId=check_id,
            InstanceId=instance_id,
        )
        while checkoutput['Status'] != "Success" and checkoutput['Status'] != "Failed":
            time.sleep(1)
            checkoutput = ssm_client.get_command_invocation(
                CommandId=check_id,
                InstanceId=instance_id,
            )
        if checkoutput['Status'] != "Success":
            print(Bcolors.WARNING+"no "+Bcolors.ORANGE+"config"+Bcolors.WARNING+" in /home/"+username+"/.ssh/"+Bcolors.ENDC)
            print("copy local config to EC2")
            if os.path.isfile(os.path.join(home, ".ssh", "config")):
                sshconfig = os.path.join(home, ".ssh", "config")
                cmd = "scp -i "+pem+" "+sshconfig+" "+username+"@"+instance_ip+":/home/"+username+"/.ssh/"
                print(Bcolors.OKGREEN+cmd+Bcolors.ENDC)
                subprocess.call(cmd, shell=True)
            else:
                print(Bcolors.WARNING+"no config in ~/.ssh/"+Bcolors.ENDC)
                print("please configure your local ssh to connect to your git providers before using bunker.")
                print()
                exit()

        # check for .ssh/keys
        if gitkeys_arr[0] != "":
            for keyfile in gitkeys_arr:
                exploded = keyfile.split("/")
                kf = exploded[-1]
                response = ssm_client.send_command(
                    InstanceIds=[instance_id],
                    DocumentName="AWS-RunShellScript",
                    Parameters={'commands': ['ls /home/'+username+'/.ssh/'+kf]}, )
                check_id = response['Command']['CommandId']
                time.sleep(1)
                checkoutput = ssm_client.get_command_invocation(
                    CommandId=check_id,
                    InstanceId=instance_id,
                )
                while checkoutput['Status'] != "Success" and checkoutput['Status'] != "Failed":
                    time.sleep(1)
                    checkoutput = ssm_client.get_command_invocation(
                        CommandId=check_id,
                        InstanceId=instance_id,
                    )
                if checkoutput['Status'] != "Success":
                    print(Bcolors.WARNING+"no "+Bcolors.ORANGE+kf+Bcolors.WARNING+" in /home/"+username+"/.ssh/"+Bcolors.ENDC)
                    print("copy local "+kf+" to EC2")
                    cmd = "scp -i "+pem+" "+keyfile+" "+username+"@"+instance_ip+":/home/"+username+"/.ssh/"
                    print(Bcolors.OKGREEN+cmd+Bcolors.ENDC)
                    subprocess.call(cmd, shell=True)

        # declare outputs dict
        outputs = {}
        for d in dirs:
            # loop thru repos (d), check if repo dir exists, then attempt to clone
            gitconfig = d+"/.git/config"
            catcher = False
            # getting repo url from .git/config
            with open(gitconfig, 'r') as gconfig:
                for line in gconfig:
                    if catcher:
                        repourl = line
                        break
                    if "[remote \"origin\"]" in line:
                        catcher = True
            repourl = repourl.strip(' \t\n\r')
            repoarr = repourl.split(" = ")
            repourl = repoarr[1]
            repourl_arr = repourl.split("/")
            newrepourl = ""
            repo_trigger = False
            for part in repourl_arr:
                if repo_trigger or ".com" in part or ".org" in part:
                    if "@" in part:
                        exp = part.split("@")
                        part = exp[1]
                    if ".com" in part or ".org" in part:
                        newrepourl += part+":"
                    else:
                        newrepourl += part+"/"
                    repo_trigger = True
            repourl = "git@"+newrepourl[:-1]
            exploded_d = d.split("/")
            newd = ""
            for thing in exploded_d:
                if thing not in similarities:
                    newd += thing+"/"
            if newd[0] != "/":
                newd = "/"+newd
            # asdf
            response = ssm_client.send_command(
                InstanceIds=[instance_id],
                DocumentName="AWS-RunShellScript",
                Parameters={'commands': ['ls /home/'+username+newd]}, )
            check_id = response['Command']['CommandId']
            # wait for response
            time.sleep(1)
            checkoutput = ssm_client.get_command_invocation(
                CommandId=check_id,
                InstanceId=instance_id,
            )
            while checkoutput['Status'] != "Success" and checkoutput['Status'] != "Failed":
                time.sleep(1)
                checkoutput = ssm_client.get_command_invocation(
                    CommandId=check_id,
                    InstanceId=instance_id,
                )
            if checkoutput['Status'] != "Success":
                response = ssm_client.send_command(
                    InstanceIds=[instance_id],
                    DocumentName="AWS-RunShellScript",
                    Parameters={'commands': ['sudo -u '+username+' git clone '+repourl+' /home/'+username+newd, 'chown -R '+username+':'+username+' /home/'+username+newd]}, )
                command_id = response['Command']['CommandId']
                outputs[command_id] = 'sudo -u '+username+' git clone '+repourl+' /home/'+username+newd, 'chown -R '+username+':'+username+' /home/'+username+newd
                print(Bcolors.YELLOW+"cloning repo: "+d+Bcolors.ENDC)
                time.sleep(1)
            else:
                print(Bcolors.OKGREEN+"repo exists: "+newd+Bcolors.ENDC)
                print(Bcolors.GREY+"we should probably pull or something?"+Bcolors.ENDC)
        if len(dirs) < 5:
            print(Bcolors.PALEYELLOW+"waiting for clone."+Bcolors.ENDC)
            countdown(10)
        # loop thru output and print any errors
        for key, value in outputs.items():
            cmdoutput = ssm_client.get_command_invocation(
                CommandId=key,
                InstanceId=instance_id,
            )
            if cmdoutput['Status'] != "Success":
                print(Bcolors.RED+"Warning - "+Bcolors.YELLOW+str(value)+Bcolors.ENDC)
                print(Bcolors.RED+"Command output ===\n"+Bcolors.YELLOW+cmdoutput['StandardErrorContent']+Bcolors.ENDC)
                print(Bcolors.RED+'===\nCloning Repo: '+cmdoutput['Status']+". "+Bcolors.YELLOW+"We will still attemp to rsync files"+Bcolors.ENDC)
        #
        #
        # backup hidden files and configs below
        #
        #
        print("cleaning up...")
        response = ssm_client.send_command(
            InstanceIds=[instance_id],
            DocumentName="AWS-RunShellScript",
            Parameters={'commands': ['chown -R '+username+':'+username+' /home/'+username]}, )
        command_id = response['Command']['CommandId']
        time.sleep(1)
        files = []
        with open(ignored, 'r') as f:
            for line in f:
                ignore = line.strip()
                files.append(ignore)
        for d in dirs:
            os.chdir(d)
            exploded_d = d.split("/")
            newd = ""
            for thing in exploded_d:
                if thing not in similarities:
                    newd += thing+"/"
            if newd[0] != "/":
                newd = "/"+newd
            print(Bcolors.OKGREEN+"\n - ", newd+"\n"+Bcolors.ENDC)
            for i in files:
                exists = subprocess.run("find . -name '"+i+"'", shell=True, stdout=subprocess.PIPE)
                exists = exists.stdout.decode('utf-8')
                if len(exists) > 1:
                    print(Bcolors.YELLOW+i+Bcolors.ENDC)
                    os.system("find . -name '"+i+"' | xargs -n1 -J % rsync -Ravze 'ssh -i "+pem+"' % "+username+"@"+instance_ip+":/home/"+username+newd)
    else:
        if running:
            stop = subprocess.run("aws ec2 stop-instances --instance-ids "+instance_id, shell=True, stdout=subprocess.PIPE)
            print("\nshutting down")
        else:
            print("\nleaving ec2 running")
        exit()
    
    if running:
        stop = subprocess.run("aws ec2 stop-instances --instance-ids "+instance_id, shell=True, stdout=subprocess.PIPE)
        print(Bcolors.CYAN+"\nshutting down: "+instance_ip+Bcolors.ENDC)
    else:
        print(Bcolors.CYAN+"\nleaving "+instance_ip+" running"+Bcolors.ENDC)
    return

if __name__ == "__main__":
    buildbunker()



#!/usr/bin/env python

"""add or remove file from ignores file"""

import configparser
import sys
import os
from os.path import expanduser
import subprocess

class Bcolors:
    """console colors"""
    CYAN = '\033[96m'
    MAGENTA = '\033[95m'
    GREY = '\033[90m'
    OKBLUE = '\033[94m'
    OKGREEN = '\033[92m'
    YELLOW = '\033[33m'
    RED = '\033[31m'
    WARNING = '\033[93m'
    FAIL = '\033[91m'
    ENDC = '\033[0m'
    BOLD = '\033[1m'
    UNDERLINE = '\033[4m'
    ORANGE = '\033[38;5;208m'
    PINK = '\033[38;5;212m'
    PALEYELLOW = '\033[38;5;228m'
    PALEBLUE = '\033[38;5;111m'
    GOLD = '\033[38;5;178m'

def query_yes_no(question, default="yes"):
    '''confirm or decline'''
    valid = {"yes": True, "y": True, "ye": True, "no": False, "n": False}
    if default is None:
        prompt = " [y/n] "
    elif default == "yes":
        prompt = " [Y/n] "
    elif default == "no":
        prompt = " [y/N] "
    else:
        raise ValueError("invalid default answer: '%s'" % default)
    while True:
        sys.stdout.write(question + prompt)
        choice = input().lower()
        if default is not None and choice == '': 
            return valid[default]
        elif choice in valid:
            return valid[choice]
        else:
            sys.stdout.write("\nPlease respond with 'yes' or 'no' (or 'y' or 'n').\n")

def ignore(ignores, yes=None, backup=None):
    """add or remove file from ignores"""
    #config here
    home = expanduser('~')
    bunker_config = os.path.join(home, ".config-bunker.ini")
    config_exists = False
    if os.path.exists(bunker_config):
        config = configparser.ConfigParser()
        config.read(bunker_config)
        config_exists = True
        prefix = config['default']['prefix']
        ignore_file = config['default']['ignored']
    else:
        print(Bcolors.PALEYELLOW+"no config file found."+Bcolors.ENDC)
        print("please run: "+Bcolors.PINK+"bunker init"+Bcolors.ENDC)
        print()
        exit()
    # read ignore file into array
    if os.path.exists(prefix+ignore_file):
        with open(prefix+ignore_file) as f:
            ignorefile_arr = f.readlines()
        ignorefile_arr = [x.strip() for x in ignorefile_arr]
    else:
        ignorefile_arr = []
    # loop sanitized list and add and remove items ass needed
    for r in ignores:
        if r in ignorefile_arr:
            if yes or query_yes_no("remove "+Bcolors.PALEYELLOW+r+Bcolors.ENDC+" from "+Bcolors.PALEBLUE+ignore_file+"?"+Bcolors.ENDC, default="yes"):
                ignorefile_arr.remove(r)
                print(Bcolors.FAIL+r+Bcolors.ENDC)
        else:
            ignorefile_arr.append(r)
            print(Bcolors.OKGREEN+r+Bcolors.ENDC)
    # finally, backup and over-write editted array to ignore file
    if backup:
        backup_ignore = prefix+ignore_file+".bak"
        print(Bcolors.PALEBLUE+"backing up ignore file to: "+Bcolors.PALEYELLOW+backup_ignore+Bcolors.ENDC)
        bcmd = "cp "+prefix+ignore_file+" "+backup_ignore
        subprocess.call(bcmd, shell=True)
    newif = ""
    print("---")
    print(Bcolors.OKGREEN+"over-writing "+ignore_file+Bcolors.ENDC)
    for r in ignorefile_arr:
        newif += r+"\n"
    f = open(prefix+ignore_file, "w+")
    f.write(newif)
    f.close()

if __name__ == "__main__":
    ignore()



#!/usr/bin/env python

"""init bunker config file"""

import configparser
import sys
import os
from os.path import expanduser
import inquirer

class Bcolors:
    """console colors"""
    CYAN = '\033[96m'
    MAGENTA = '\033[95m'
    GREY = '\033[90m'
    OKBLUE = '\033[94m'
    OKGREEN = '\033[92m'
    YELLOW = '\033[33m'
    RED = '\033[31m'
    WARNING = '\033[93m'
    FAIL = '\033[91m'
    ENDC = '\033[0m'
    BOLD = '\033[1m'
    UNDERLINE = '\033[4m'
    ORANGE = '\033[38;5;208m'
    PINK = '\033[38;5;212m'
    PALEYELLOW = '\033[38;5;228m'
    PALEBLUE = '\033[38;5;111m'
    GOLD = '\033[38;5;178m'

def query_yes_no(question, default="yes"):
    '''confirm or decline'''
    valid = {"yes": True, "y": True, "ye": True, "no": False, "n": False}
    if default is None:
        prompt = " [y/n] "
    elif default == "yes":
        prompt = " [Y/n] "
    elif default == "no":
        prompt = " [y/N] "
    else:
        raise ValueError("invalid default answer: '%s'" % default)
    while True:
        sys.stdout.write(question + prompt)
        choice = input().lower()
        if default is not None and choice == '': 
            return valid[default]
        elif choice in valid:
            return valid[choice]
        else:
            sys.stdout.write("\nPlease respond with 'yes' or 'no' (or 'y' or 'n').\n")

def init_bunker(instance_id=None):
    #config here
    home = expanduser('~')
    bunker_config = os.path.join(home, ".config-bunker.ini")
    config_exists = False
    gitkeys_arr = []
    if os.path.exists(bunker_config):
        config = configparser.ConfigParser()
        config.read(bunker_config)
        if "gitkeys" in config['default']:
            config_exists = True
            prefix = config['default']['prefix']
            pem = config['default']['pem']
            if instance_id is None:
                instance_id = config['default']['instance_id']
            username = config['default']['username']
            gitkeys = config['default']['gitkeys']
            keycount = gitkeys.count(', ')
            if keycount > 0:
                gitkeys_arr = config['default']['gitkeys'].split(", ")
            else:
                gitkeys_arr.append(config['default']['gitkeys'])
    if not config_exists:
        print(Bcolors.PALEYELLOW+"deprecated config file or no config file found."+Bcolors.ENDC)
        prefix, pem, instance_id, username, gitkeys = home+"/", "unknown", "unknown", "ubuntu", "unknown"

    newconfig = """[default]
prefix = {prefix}
repos = {repos}
ignored = {ignored}
pem = {pem}
instance_id = {iid}
username = {username}
gitkeys = {gitkeys}
"""
    instance_id = input(Bcolors.ENDC+"EC2 Instance ID ["+Bcolors.UNDERLINE+instance_id+Bcolors.ENDC+"] : "+Bcolors.PALEYELLOW) or instance_id
    while True:
        prefix = input(Bcolors.ENDC+"Directory where you will store repositories file ["+Bcolors.UNDERLINE+prefix+Bcolors.ENDC+"] : "+Bcolors.PALEYELLOW) or prefix
        if not prefix.endswith("/"):
            if config['default']['prefix']:
                prefix = config['default']['prefix']
            else:
                prefix = "unknown"
            print(Bcolors.FAIL+"ERROR: "+Bcolors.WARNING+"prefix must end with a /"+Bcolors.ENDC)
            continue
        elif not os.path.isdir(prefix):
            if config['default']['prefix']:
                prefix = config['default']['prefix']
            else:
                prefix = home+"/"
            print(Bcolors.FAIL+"ERROR: "+Bcolors.WARNING+"directory does not exist"+Bcolors.ENDC)
            continue
        else:
            break
    while True:
        repofile = input(Bcolors.ENDC+"Name of repository file ["+Bcolors.UNDERLINE+"repositories.txt"+Bcolors.ENDC+"] : "+Bcolors.PALEYELLOW) or "repositories.txt"
        if not repofile.endswith(".txt"):
            repofile = "repositories.txt"
            print(Bcolors.FAIL+"ERROR: "+Bcolors.WARNING+"repository filename must be .txt"+Bcolors.ENDC)
            continue
        else:
            break
    while True:
        ignorefile = input(Bcolors.ENDC+"Name of ignored-git-files file ["+Bcolors.UNDERLINE+"ignoredfiles.txt"+Bcolors.ENDC+"] : "+Bcolors.PALEYELLOW) or "ignoredfiles.txt"
        if not ignorefile.endswith(".txt"):
            ignorefile = "ignoredfiles.txt"
            print(Bcolors.FAIL+"ERROR: "+Bcolors.WARNING+"ignored filename must be .txt"+Bcolors.ENDC)
            continue
        else:
            break
    while True:
        pem = input(Bcolors.ENDC+"Full path to .pem keyfile ["+Bcolors.UNDERLINE+pem+Bcolors.ENDC+"] : "+Bcolors.PALEYELLOW) or pem
        if not pem.endswith(".pem"):
            if config['default']['pem']:
                pem = config['default']['pem']
            else:
                pem = "unknown"
            print(Bcolors.FAIL+"ERROR: "+Bcolors.WARNING+"keyfile name must be .pem"+Bcolors.ENDC)
            continue
        elif not os.path.isfile(pem):
            if config['default']['pem']:
                pem = config['default']['pem']
            else:
                pem = "unknown"
            print(Bcolors.FAIL+"ERROR: "+Bcolors.WARNING+"file does not exist"+Bcolors.ENDC)
            continue
        else:
            break
    while True:
        username = input(Bcolors.ENDC+"EC2 username ["+Bcolors.UNDERLINE+username+Bcolors.ENDC+"] : ") or username
        if username != "ubuntu":
            print(Bcolors.FAIL+"ERROR: "+Bcolors.WARNING+"bunker currently only supports ubuntu"+Bcolors.ENDC)
            continue
        else:
            break
    
    f = []
    d = []
    for root, dirs, files in os.walk(os.path.join(home, ".ssh"), topdown=False):
        for name in files:
            if ".pub" in name:
                name = name.replace(".pub", "")
                if os.path.join(root, name) in gitkeys_arr:
                    d.append(os.path.join(root, name))
                f.append(os.path.join(root, name))
    questions = [inquirer.Checkbox(
        'keyfiles',
        message="Select key files (use arrow keys) to copy to the EC2.\n"+Bcolors.GOLD+"NOTE: "+Bcolors.ENDC+"Select only keys you need for access to your git provider(s)",
        choices=f,
        default=d,
    )]
    answers = inquirer.prompt(questions)  # returns a dict
    if len(answers['keyfiles']) < 1:
        print(Bcolors.FAIL+"WARNING: "+Bcolors.WARNING+"you did not select a key file, so bunker build will not be able to clone your git repos."+Bcolors.ENDC)
        print("if this was not your intention, please re-run: "+Bcolors.PINK+"bunker init"+Bcolors.ENDC)
        newkeyfiles = []
    else:
        newkeyfiles = answers['keyfiles']
    newgitkeys = ""
    for item in newkeyfiles:
        newgitkeys += item+", "
    newgitkeys = newgitkeys[:-2]
    print(Bcolors.OKGREEN+"bunker is now configured."+Bcolors.ENDC+" writing: "+Bcolors.PALEBLUE+"~/.config-bunker.ini"+Bcolors.ENDC)
    print("if your ec2 is up and running, try: "+Bcolors.PINK+"bunker install"+Bcolors.ENDC)
    newconfig = newconfig.format(prefix=prefix, repos=repofile, ignored=ignorefile, pem=pem, iid=instance_id, username=username, gitkeys=newgitkeys) 
    with open(bunker_config, 'w+') as f:
        f.write(newconfig)
    exit()

if __name__ == "__main__":
    init_bunker()



#!/usr/bin/env python

"""bunker is a command line program for setting up an ec2 in AWS for remote development or as a backup. It can clone your git repos, and transfer ignored files from your machine to the ec2."""

import argparse
from argparse import RawTextHelpFormatter as rawtxt
import sys
import signal
import time
import os
from os.path import expanduser
from datetime import datetime
from pathlib import Path
import pkg_resources

#from init import init_bunker
#from install import install
#from prompt import prompt
#from buildbunker import buildbunker
#from repo import repo
#from ignore import ignore
from .init import init_bunker
from .install import install
from .prompt import prompt
from .buildbunker import buildbunker
from .repo import repo
from .ignore import ignore

def signal_handler(sig, frame):
    """handle control c"""
    print(Bcolors.WARNING+"\nExit with "+Bcolors.FAIL+"Control C\n"+Bcolors.ENDC)
    sys.exit(0)
signal.signal(signal.SIGINT, signal_handler)

def query_yes_no(question, default="yes"):
    '''confirm or decline'''
    valid = {"yes": True, "y": True, "ye": True, "no": False, "n": False}
    if default is None:
        prompt = " [y/n] "
    elif default == "yes":
        prompt = " [Y/n] "
    elif default == "no":
        prompt = " [y/N] "
    else:
        raise ValueError("invalid default answer: '%s'" % default)
    while True:
        sys.stdout.write(question + prompt)
        choice = input().lower()
        if default is not None and choice == '':
            return valid[default]
        elif choice in valid:
            return valid[choice]
        else:
            sys.stdout.write("\nPlease respond with 'yes' or 'no' (or 'y' or 'n').\n")

def is_tool(name):
    """Check whether `name` is on PATH and marked as executable."""
    from shutil import which
    return which(name) is not None

class Bcolors:
    """console colors"""
    CYAN = '\033[96m'
    MAGENTA = '\033[95m'
    GREY = '\033[90m'
    OKBLUE = '\033[94m'
    OKGREEN = '\033[92m'
    YELLOW = '\033[33m'
    RED = '\033[31m'
    WARNING = '\033[93m'
    FAIL = '\033[91m'
    ENDC = '\033[0m'
    BOLD = '\033[1m'
    UNDERLINE = '\033[4m'
    ORANGE = '\033[38;5;208m'
    PINK = '\033[38;5;212m'
    PALEYELLOW = '\033[38;5;228m'
    PALEBLUE = '\033[38;5;111m'
    GOLD = '\033[38;5;178m'

def bunker():
    """bunker is a command line program for setting up an ec2 in AWS for remote development or as a backup. It can clone your git repos, and transfer ignored files from your machine to the ec2."""
    version = pkg_resources.require("aws-bunker")[0].version
    parser = argparse.ArgumentParser(
        description='bunker is a command line program for setting up an ec2 in AWS for remote development or as a backup. It can clone your git repos, and transfer ignored files from your machine to the ec2.',
        prog='bunker',
        formatter_class=rawtxt
    )
    parser.add_argument('-v', '--version', action='version', version='%(prog)s '+version)
    
    subparsers = parser.add_subparsers(title='subcommands', required=True, dest="subcommand", help="""for more info: type `bunker subcommand -h`

the """+Bcolors.PINK+"""install"""+Bcolors.ENDC+""", """+Bcolors.PINK+"""extra """+Bcolors.ENDC+"""and """+Bcolors.PINK+"""prompt """+Bcolors.ENDC+"""subcommands require custom bash scripts to exist on your EC2. The scripts also need to be executable.
for example scripts please visit the project homepage:
"""+Bcolors.GOLD+"""https://gitlab.com/shindagger/bunker **"""+Bcolors.ENDC)
    #parser.print_help()
    
    parser_init = subparsers.add_parser('init', formatter_class=argparse.RawTextHelpFormatter, help='init bunker by writing ~/.config-bunker', description="init bunker by writing ~/.config-bunker")
    parser_init.set_defaults(func=init_bunker)
    parser_install = subparsers.add_parser('install', formatter_class=argparse.RawTextHelpFormatter, help="""install essential software
"""+Bcolors.GOLD+"""default: `install-essentials.sh`"""+Bcolors.ENDC, description="""install essential software
default: """+Bcolors.GOLD+"""`install-essentials.sh`"""+Bcolors.ENDC+""" on the EC2.
if the file is not executable use the """+Bcolors.OKGREEN+"""[-e]"""+Bcolors.ENDC+""" flag.
for example files please visit: """+Bcolors.PALEBLUE+"""https://gitlab.com/shindagger/bunker"""+Bcolors.ENDC)
    parser_install.add_argument('-y', '--yes', action='store_true', help='auto approve installing essentials.')
    parser_install.add_argument('-s', action='store_true', help='startup EC2 first.')
    parser_install.add_argument('run', nargs='?', default='/home/ubuntu/install-essentials.sh', help='run provided shell script. please use absolute paths. default: '+Bcolors.PALEBLUE+'`/home/ubuntu/install-essentials.sh`'+Bcolors.ENDC)
    parser_install.add_argument('-e', action='store_true', help='make shell script executable first.')
    parser_install.add_argument('-b', '--bucket', nargs=3, metavar=('BUCKET','REGION','PREFIX'), default=['none', 'none', 'none'], help='save standard out to an s3 bucket (for trouble-shooting). takes 3 arguments: '+Bcolors.PALEBLUE+'bucket, region, and prefix'+Bcolors.ENDC+'\nexample: '+Bcolors.PALEYELLOW+'bunker install /home/ubuntu/script.sh -b bucketname us-east-1 ssm-test'+Bcolors.ENDC)
    parser_install.set_defaults(func=install)
    parser_prompt = subparsers.add_parser('prompt', formatter_class=argparse.RawTextHelpFormatter, help="""source file in /home/ubuntu/.profile
"""+Bcolors.GOLD+"""default: `/home/ubuntu/.prompt.sh`"""+Bcolors.ENDC, description="""source file in /home/ubuntu/.profile
default: """+Bcolors.GOLD+"""`/home/ubuntu/.prompt.sh`"""+Bcolors.ENDC+"""
for an example file please visit: """+Bcolors.PALEBLUE+"""https://gitlab.com/shindagger/bunker"""+Bcolors.ENDC)
    parser_prompt.add_argument('file', nargs='?', default='/home/ubuntu/.prompt.sh', help='source provided file. please use absolute paths. default: '+Bcolors.PALEBLUE+'`/home/ubuntu/.prompt.sh`'+Bcolors.ENDC)
    parser_prompt.add_argument('-y', '--yes', action='store_true', help='auto approve customizing bash prompt.')
    parser_prompt.add_argument('-b', '--bucket', nargs=3, metavar=('BUCKET','REGION','PREFIX'), default=['none', 'none', 'none'], help='save standard out to an s3 bucket (for trouble-shooting). takes 3 arguments: '+Bcolors.PALEBLUE+'bucket, region, and prefix'+Bcolors.ENDC+'\nexample: '+Bcolors.PALEYELLOW+'bunker prompt /home/ubuntu/custom-bash-prompt.sh -b bucketname us-east-1 ssm-test'+Bcolors.ENDC)
    parser_prompt.add_argument('-s', action='store_true', help='startup EC2 first.')
    parser_prompt.set_defaults(func=prompt)
    parser_repo = subparsers.add_parser('repo', formatter_class=argparse.RawTextHelpFormatter, help="""add or remove repos to/from the repo file.
if the repo is in the file it will remove the repo.
if the repo is not in the file, it will be added.
"""+Bcolors.PALEBLUE+"""both relative and absolute paths will work."""+Bcolors.ENDC, description="""add or remove repos to/from the repo file.
if the repo is in the file it will be removed.
if the repo is not in the file, it will be added.
"""+Bcolors.PALEBLUE+"""both relative and absolute paths will work."""+Bcolors.ENDC)
    parser_repo.add_argument('repos', nargs='+', help='a list of repos to add or remove to/from the repo file. requires at least one repo.')
    parser_repo.add_argument('-y', '--yes', action='store_true', help='approve all prompts as yes.')
    parser_repo.add_argument('-b', '--backup', action='store_true', help='backup repo file before over-writing.')
    parser_repo.set_defaults(func=repo)
    parser_ignore = subparsers.add_parser('ignore', formatter_class=argparse.RawTextHelpFormatter, help="""add or remove filenames to/from the ignore file.
if the filename is in the ignore file it will be removed.
if the filename is not in the ignore file, it will be added.""", description="""add or remove filename to/from the ignore file.
if the filename is in the ignore file it will be removed.
if the filename is not in the ignore file, it will be added.""")
    parser_ignore.add_argument('ignores', nargs='+', help='a list of filenames to add or remove to/from the ignore file. requires at least one file to ignore.')
    parser_ignore.add_argument('-y', '--yes', action='store_true', help='approve all prompts as yes.')
    parser_ignore.add_argument('-b', '--backup', action='store_true', help='backup ignore file before over-writing.')
    parser_ignore.set_defaults(func=ignore)
    parser_build = subparsers.add_parser('build', help='clone git repos and copy over ignored files', description="clone git repos and copy over ignored files")
    parser_build.add_argument('-y', '--yes', action='store_true', help='auto approve cloning repos and backing up files.')
    parser_build.add_argument('-S', action='store_true', help='stop EC2 after running.')
    parser_build.add_argument('-s', action='store_true', help='startup EC2 first.')
    parser_build.add_argument('-r', '--repo', nargs='?', default=None, help='Source directory to be duplicated to remote bunker')
    parser_build.set_defaults(func=buildbunker)
    args = parser.parse_args()
    # error checking
    if args.subcommand != 'init':
        home = expanduser('~')
        bunker_config = os.path.join(home, ".config-bunker.ini")
        if not os.path.isfile(bunker_config):
            print(Bcolors.WARNING+"no configuration file exists at: "+Bcolors.ORANGE+bunker_config+Bcolors.ENDC)
            print("please run:")
            print()
            print(Bcolors.PINK+"   bunker init"+Bcolors.ENDC)
            print()
            time.sleep(1)
            print(Bcolors.GREY+"... bunker --help ..."+Bcolors.ENDC)
            time.sleep(1)
            parser.print_help()
            exit()
    # auf gehts
    if args.subcommand == 'build':
        yes = args.yes
        running = args.S
        dostart = args.s
        source = args.repo
        args.func(running, dostart, source, yes)
    elif args.subcommand == 'install':
        yes = args.yes
        dostart = args.s
        run = args.run
        execute = args.e
        bucket_info = args.bucket
        args.func(dostart=dostart, yes=yes, run=run, execute=execute, bucket_info=bucket_info)
    elif args.subcommand == 'prompt':
        yes = args.yes
        dostart = args.s
        run = args.file
        bucket_info = args.bucket
        args.func(dostart=dostart, yes=yes, run=run, bucket_info=bucket_info)
    elif args.subcommand == 'repo':
        yes = args.yes
        repos = args.repos
        backup = args.backup
        args.func(repos, yes, backup)
    elif args.subcommand == 'ignore':
        yes = args.yes
        ignores = args.ignores
        backup = args.backup
        args.func(ignores, yes, backup)
    else:
        args.func()

if __name__ == "__main__":
    bunker()
